@@ -10,12 +10,10 @@ import kate
 from compiler import parse
 
 from PyKDE4.ktexteditor import KTextEditor
-from PyKDE4.kdeui import KIcon
 from PyQt4 import QtCore
-from PyQt4.QtCore import QSize
-from PyQt4.QtCore import QModelIndex, Qt, QVariant
 from pysmell.codefinder import CodeFinder
 
+from autopate import AbstractCodeCompletionModel
 
 global modules_path
 global python_path
@@ -51,96 +49,30 @@ import_begin = re.compile(_import_begin + "$")
 import_complete = re.compile(_import_complete + "?$")
 
 
-class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
+class PythonCodeCompletionModel(AbstractCodeCompletionModel):
 
     MIMETYPES = ['', 'py', 'pyc']
-
-    def __init__(self, *args, **kwargs):
-        super(PythonCodeCompletionModel, self).__init__(*args, **kwargs)
-        self.resultList = []
-
-    roles = {
-        KTextEditor.CodeCompletionModel.CompletionRole:
-            QVariant(
-                KTextEditor.CodeCompletionModel.FirstProperty |
-                KTextEditor.CodeCompletionModel.Public |
-                KTextEditor.CodeCompletionModel.LastProperty |
-                KTextEditor.CodeCompletionModel.Prefix),
-        KTextEditor.CodeCompletionModel.ScopeIndex:
-            QVariant(0),
-        KTextEditor.CodeCompletionModel.MatchQuality:
-            QVariant(10),
-        KTextEditor.CodeCompletionModel.HighlightingMethod:
-            QVariant(QVariant.Invalid),
-        KTextEditor.CodeCompletionModel.InheritanceDepth:
-            QVariant(0),
-    }
+    OPERATORS = ["=", " ", "[", "]", "(", ")", "{", "}", ":", ">", "<",
+                 "+", "-", "*", "/", "%", " and ", " or ", ","]
 
     def completionInvoked(self, view, word, invocationType):
+        line = super(PythonCodeCompletionModel, self).completionInvoked(view,
+                                                                        word,
+                                                                        invocationType)
         is_auto = False
-        line_start = word.start().line()
-        line_end = word.end().line()
-        self.resultList = []
-        self.invocationType = invocationType
-        path = unicode(view.document().url().path())
-        if line_start != line_end:
-            return
-        if not path.split(".")[-1] in self.MIMETYPES:
-            return
-        doc = view.document()
-        line = unicode(doc.line(line_start))
-        if not line:
-            return
-        line = self._parse_line(line)
         line_rough = line
         if 'from' in line or 'import' in line:
             is_auto = self.autoCompleteImport(view, word, line)
-        line = self._get_expression_last_expression(line)
+        line = self.get_expression_last_expression(line)
         if not is_auto and line:
             is_auto = self.autoCompleteDynamic(view, word, line)
-        if not is_auto and line and line_rough and not '.' in line_rough:
+        if not is_auto and line and line_rough and not self.SEPARATOR in line_rough:
             is_auto = self.autoCompleteInThisFile(view, word, line)
 
-    #http://api.kde.org/4.5-api/kdelibs-apidocs/interfaces/ktexteditor/html/classKTextEditor_1_1CodeCompletionModel.html#3bd60270a94fe2001891651b5332d42b
-    def index(self, row, column, parent):
-        if (row < 0 or row >= len(self.resultList) or
-            column < 0 or column >= KTextEditor.CodeCompletionModel.ColumnCount or
-            parent.isValid()):
-            return QModelIndex()
-        return self.createIndex(row, column)
-
-    def rowCount(self, parent):
-        if parent.isValid():
-            return 0  # Do not make the model look hierarchical
-        else:
-            return len(self.resultList)
-
-    def data(self, index, role, *args, **kwargs):
-        item = self.resultList[index.row()]
-        if index.column() == KTextEditor.CodeCompletionModel.Name:
-            if role == Qt.DisplayRole:
-                return QVariant(item['text'])
-            try:
-                return self.roles[role]
-            except KeyError:
-                pass
-        elif index.column() == KTextEditor.CodeCompletionModel.Icon:
-            if role == Qt.DecorationRole:
-                return QVariant(KIcon(item["icon"]).pixmap(QSize(16, 16)))
-        elif index.column() == KTextEditor.CodeCompletionModel.Arguments:
-            item_args = item.get("args", None)
-            if role == Qt.DisplayRole and item_args:
-                return QVariant(item_args)
-        elif index.column() == KTextEditor.CodeCompletionModel.Postfix:
-            item_description = item.get("description", None)
-            if role == Qt.DisplayRole and item_description:
-                return QVariant(item_description)
-        return QVariant()
-
     def executeCompletionItem(self, doc, word, row):
         raw, col = word.start().position()
         line = unicode(doc.line(raw))
-        line = self._parse_line(line)
+        line = self.parse_line(line)
         t = self.resultList[row].get('type', None)
         args = self.resultList[row].get('args', None).strip()
         text = self.resultList[row].get('text', None)
@@ -165,7 +97,7 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
             if not submodules:
                 submodules = []
             else:
-                submodules = submodules.split('.')[:-1]
+                submodules = submodules.split(self.SEPARATOR)[:-1]
             self.resultList = self.getSubmodules(module,
                                                   submodules,
                                                   attributes=False)
@@ -174,7 +106,7 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
         if mfc:
             module, submodules, import_module = mfc.groups()
             if submodules:
-                submodules = submodules.split('.')
+                submodules = submodules.split(self.SEPARATOR)
             else:
                 submodules = []
             self.resultList = self.getSubmodules(module,
@@ -185,7 +117,7 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
 
     def autoCompleteDynamic(self, view, word, line):
         try:
-            last_dot = line.rindex('.')
+            last_dot = line.rindex(self.SEPARATOR)
             line = line[:last_dot]
         except ValueError:
             pass
@@ -196,23 +128,14 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
         text = self._parse_text(view, word, line)
         return self.getTextInfo(text, self.resultList)
 
-    def _parse_line(self, line):
-        line = line.strip()
+    def parse_line(self, line):
+        line = super(PythonCodeCompletionModel, self).parse_line(line)
         if "'" in line or '"' in line:
             return line
         if ";" in line:
-            return self._parse_line(line.split(";")[-1])
+            return self.parse_line(line.split(";")[-1])
         return line
 
-    def _get_expression_last_expression(self, line):
-        operators = ["=", " ", "[", "]", "(", ")", "{", "}", ":", ">", "<",
-                     "+", "-", "*", "/", "%", "and", "or"]
-        opmax = max(operators, key=lambda e: line.rfind(e))
-        opmax_index = line.rfind(opmax)
-        if line.find(opmax) != -1:
-            line = line[opmax_index + 1:]
-        return line.strip()
-
     def _parse_text(self, view, word, line):
         doc = view.document()
         text_list = unicode(doc.text()).split("\n")
@@ -290,7 +213,7 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
         try:
             code = parse(text)
             code_walk = compiler.walk(code, CodeFinder())
-            code_line_split = code_line.split('.')
+            code_line_split = code_line.split(self.SEPARATOR)
             prefix = code_line_split[0]
             prfx_code_line = '%s%s' % (PYSMELL_PREFIX, prefix)
             if prfx_code_line in code_walk.modules['CONSTANTS']:
@@ -306,8 +229,8 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
                 return True
             elif code_walk.modules['POINTERS'].get(prfx_code_line, None):
                 module_path = code_walk.modules['POINTERS'].get(prfx_code_line, None)
-                module = module_path.split('.')[0]
-                submodules = module_path.split('.')[1:]
+                module = module_path.split(self.SEPARATOR)[0]
+                submodules = module_path.split(self.SEPARATOR)[1:]
                 submodules.extend(code_line_split[1:])
                 module_done, submodules_undone = self.getModuleSmart(module, submodules)
                 if not submodules_undone:
@@ -316,7 +239,7 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
                         if autocompletion_submodule not in self.resultList:
                             self.resultList.append(autocompletion_submodule)
                 submodules_undone.reverse()
-                line = '.'.join(submodules_undone)
+                line = self.SEPARATOR.join(submodules_undone)
                 text = module_done.get_source()
                 if line:
                     return self.getDynamic(text, line)
@@ -342,28 +265,6 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
             pass
         return python_path
 
-    @classmethod
-    def createItemAutoComplete(cls, text,
-                                    icon='unknown',
-                                    args=None,
-                                    description=None,
-                                    place='code'):
-        icon_converter = {'package': 'code-block',
-                          'module': 'code-context',
-                          'unknown': 'unknown',
-                          'constant': 'code-variable',
-                          'class': 'code-class',
-                          'function': 'code-function'}
-        max_description = 50
-        if description and len(description) > max_description:
-            description = description.strip()
-            description = '%s...' % description[:max_description]
-        return {'text': text,
-                'icon': icon_converter[icon],
-                'args': args or '',
-                'type': icon,
-                'description': description or ''}
-
     def getTextInfo(self, text, list_autocomplete, line=None):
         try:
             code = parse(text)
@@ -408,15 +309,14 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
         line = e.lineno
         message = 'There was a syntax error in this file:'
         if f:
-            message = '%s\n  * file: %s' %(message, f)
+            message = '%s\n  * file: %s' % (message, f)
         if text:
-            message = '%s\n  * text: %s' %(message, text)
+            message = '%s\n  * text: %s' % (message, text)
         if line:
-            message = '%s\n  * line: %s' %(message, line)
+            message = '%s\n  * line: %s' % (message, line)
         kate.gui.popup(message,
                        2, icon='dialog-warning', minTextWidth=200)
 
-
     def treatment_pysmell_const(self, constant):
         constant = constant.replace(PYSMELL_PREFIX, '')
         return self.createItemAutoComplete(constant, 'constant')
@@ -466,7 +366,7 @@ def createSignalAutocompleteDocument(view, *args, **kwargs):
     #http://code.google.com/p/lilykde/source/browse/trunk/frescobaldi/python/frescobaldi_app/mainapp.py#1391
     #http://api.kde.org/4.0-api/kdelibs-apidocs/kate/html/katecompletionmodel_8cpp_source.html
     #https://svn.reviewboard.kde.org/r/1640/diff/?expand=1
-    PythonCodeCompletionModel.getTopLevelModules()
+    PythonCodeCompletionModel.getPythonPath()
     cci = view.codeCompletionInterface()
     cci.registerCompletionModel(codecompletationmodel)
 
